@@ -98,6 +98,7 @@ static unsigned int dst_port_seq = 0;
 static unsigned int tos = 0;
 static unsigned int flow_label = 0;
 static int noroute = 0;
+static unsigned int fwmark = 0;
 static int packet_len = -1;
 static double wait_secs = DEF_WAIT_SECS;
 static double send_secs = DEF_SEND_SECS;
@@ -516,6 +517,10 @@ static CLIF_option option_list[] = {
 	{ 0, "sport", "num", "Use source port %s for outgoing packets. "
 			    "Implies `-N 1'",
 			    set_port, &src_port, 0, CLIF_EXTRA },
+#ifdef SO_MARK
+	{ 0, "fwmark", "num", "Set firewall mark for outgoing packets",
+			    CLIF_set_uint, &fwmark, 0, 0 },
+#endif
 	{ "U", "udp", 0, "Use UDP to particular port for tracerouting "
 			    "(instead of increasing the port per each probe), "
 			    "default port is " _TEXT(DEF_UDP_PORT),
@@ -1007,6 +1012,15 @@ void tune_socket (int sk) {
 	}
 
 
+#ifdef SO_MARK
+	if (fwmark) {
+	    if (setsockopt (sk, SOL_SOCKET, SO_MARK,
+					&fwmark, sizeof (fwmark)) < 0
+	    )  error ("setsockopt SO_MARK");
+	}
+#endif
+
+
 	if (rtbuf && rtbuf_len) {
 	    if (af == AF_INET) {
 		if (setsockopt (sk, IPPROTO_IP, IP_OPTIONS,
